#!/usr/bin/env python3

from collections import defaultdict
from datetime import datetime
from random import choices
from os import listdir
from csv import DictReader

import os.path
import re
import discord

class ColdCall():
    def __init__ (self):
        self.today = str(datetime.date(datetime.now()))
        # how much less likely should it be that a student is called upon?
        self.weight = 2 

        # filenames
        self.__fn_studentinfo = "data/student_information.tsv"
        self.__fn_daily_calllist = f"data/call_list-{self.today}.tsv"
        self.__fn_daily_attendence = f"data/attendence-{self.today}.tsv"

    def __load_prev_questions(self):
        previous_questions = defaultdict(int)

        for fn in listdir("./data/"):
            if re.match("call_list-\d{4}-\d{2}-\d{2}.tsv", fn):
                with open(f"./data/{fn}", 'r') as f:
                    for row in DictReader(f, delimiter="\t"):
                        previous_questions[row["discord_name"]] += 1

        return previous_questions
    
    def __get_preferred_name(self, selected_student):
        # translate the discord name into the preferred students name,
        # if possible, otherwise return the discord name

        preferred_names = {}
        with open(self.__fn_studentinfo, 'r') as f:
            for line in f.readlines():
                x = line.strip().split("\t")
                if x[0] == "Timestamp":
                    continue

                preferred_names[x[4]] = x[2]

        if selected_student in preferred_names:
            return preferred_names[selected_student]
        else:
            return None

    def __select_student_from_list (self, students_present):
        prev_questions = self.__load_prev_questions()
        
        # created a weighted list by starting out with everybody 1
        weights = {s : 1 for s in students_present}
        
        for s in students_present:
            for i in range(0, prev_questions[s]):
                # reduce the weight by a factor of 1/weight each time the student has been called upon
                weights[s] = weights[s] / self.weight

        # choose one student from the weighted list
        print(weights)
        return choices(list(weights.keys()), weights=list(weights.values()), k=1)[0]

    def __record_attendence(self, students_present):
        # if it's the first one of the day, write it out
        if not os.path.exists(self.__fn_daily_attendence):
            with open(self.__fn_daily_attendence, "w") as f:
                print("\t".join(["timestamp", "attendence_list"]), file=f)

        # open for appending the student
        with open(self.__fn_daily_attendence, "a") as f:
            print("\t".join([str(datetime.now()),
                             ",".join(students_present)]),
                  file=f)

    def __record_coldcall(self, selected_student):
        # if it's the first one of the day, write it out
        if not os.path.exists(self.__fn_daily_calllist):
            with open(self.__fn_daily_calllist, "w") as f:
                print("\t".join(["discord_name", "timestamp", "answered", "assessment"]), file=f)

        # open for appending the student
        with open(self.__fn_daily_calllist, "a") as f:
            print("\t".join([selected_student, str(datetime.now()),
                             "MISSING", "MISSING"]), file=f)

    def coldcall(self, students_present):
        selected_student = self.__select_student_from_list(students_present)

        # record the called-upon student in the right place
        self.__record_attendence(students_present)
        self.__record_coldcall(selected_student)

        preferred_name = self.__get_preferred_name(selected_student)
        if preferred_name:
            coldcall_message = f"{preferred_name} (@{selected_student}), you're up!"
        else:
            coldcall_message = f"@{selected_student}, you're up!"
        return coldcall_message

# cc = ColdCall()

# test_student_list = ["Jordan", "Kristen Larrick", "Madison Heisterman", "Maria.Au20", "Laura (Alia) Levi", "Leona Aklipi", "Linya Feng", "anne", "kirst", "emmaaitelli", "ashleylee", "allie_partridge", "Tiana_Cole", "Hamin", "Ella Qu", "Angel Su", "Shizuka", "Ben Baird", "Kim Do", "Isaacm24", "Sam Bell", "Courtneylg", "TasnimHasan"]
# cc.coldcall(test_student_list)

# test_student_list = ["Jordan", "Kristen Larrick", "Mako"]
# cc.coldcall(test_student_list)

# test_student_list = ["Jordan", "Kristen Larrick"]
# cc.coldcall(test_student_list)
